#!/usr/bin/perl

# index2allfiles.pl
#    Based on index2html.pl by Pasi Ojala and Marko Mäkelä
#    Programmed by Marko Mäkelä <marko.makela@iki.fi>
# Last revision: June 1, 2005

# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
# License for more details.

# This script generates ALLFILES.html and ALLDIRS.html files for some
# subdirectories in the hierarchy ftp.funet.fi:/pub/cbm, or
# http://www.funet.fi/pub/cbm/.
#
# It gathers its information from the 00INDEX files that are stored in
# each directory.  See index2html.pl for a description of their format.

$COLS = 78; # max cols per line in ALLFILES

#URL expression match pattern
$URLexpression = '&lt;URL:(.+?)&gt;';

$rootpwd = `pwd`;chop $rootpwd;
$workdir = '';

dirindex ("/pub/cpm");
masterindex ("/pub/cpm");

dirindex ("/pub/cbm-pc");
masterindex ("/pub/cbm-pc");

dirindex ("/pub/cbm");
masterindex ("/pub/cbm");
masterindex ("/pub/cbm/pet");
masterindex ("/pub/cbm/vic20");
masterindex ("/pub/cbm/c128");
masterindex ("/pub/cbm/c64");
masterindex ("/pub/cbm/plus4");
#masterindex ("/pub/cbm/b");
#masterindex ("/pub/cbm/c65");
masterindex ("/pub/cbm/firmware");
masterindex ("/pub/cbm/schematics");
masterindex ("/pub/cbm/geos");
masterindex ("/pub/cbm/crossplatform");


exit(0);

# replace a file if it has changed
sub replace
{
    my ($new, $old) = @_;
    if (system ("cmp", "-s", $old, $new) >> 8)
    {
	die "could not create $old from $new: $!\n" if
	    system ("mv", $new, $old) >> 8;
	chmod 0664, $old;
    }
    else
    {
	unlink $new;
    }
}

sub masterindex
{
    ($workdir) = @_;
    local $basedir = $workdir;
    $workdir =~ s|^/pub|$rootpwd|go;
    chdir $workdir || die "Cannot chdir to $workdir!\n";

    open(HTML, ">/tmp/ALLFILES.html$$") ||
	die "Could not create /tmp/ALLFILES.html$$!\n";

    open(TEXT, ">/tmp/ALLFILES$$") ||
	die "Could not create /tmp/ALLFILES$$!\n";

    my ($uri) = '../' x (($basedir =~ tr|/|/|) - 2);
    print HTML '<?xml version="1.0"?><!-- -*- xml -*- -->
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
 "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html
 xmlns="http://www.w3.org/1999/xhtml"
 lang="en" xml:lang="en" dir="ltr">

<head>
<title>', &htmlname($basedir), ' index</title>
<link rel="stylesheet" href="', $uri,
'.index.css" />
<link rel="icon" href="', $uri,
'.icon.png" />
<link rel="SHORTCUT ICON" href="', $uri,
'.icon.ico" />
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
</head><body>

<h1>Files under <a href="index.html">', &htmlname($basedir), "</a></h1>
<div class=\"files\">\n";

    print TEXT "Files under $basedir\n\n";
    print TEXT "The HTML version is in $basedir/ALLFILES.html\n\n\n";

    &processdir("", $workdir, 0);

    print HTML "</div>
<p class=\"bot\"><a href=\"${uri}MIRRORS.html\">Mirror sites</a>\n&ndash;
<a href=\"${uri}README.html\">General information</a>\n&ndash;
<a href=\"${uri}FILETYPES.html\">File types</a>";
    print HTML "\n&ndash;
<a href=\"${uri}crossplatform/transfer/transfer.html\">Data transfer</a>"
	if $workdir =~ m|/cbm/|;
    print HTML "</p>\n";
    print HTML
"<p class=\"bot\"><small>The Commodore brandname and the chickenhead logo are
property of Commodore International BV, a Tulip company.</small></p>\n"
	if $workdir =~ m|/cbm/|;
    print HTML "</body></html>\n";
    close (HTML);
    &replace ("/tmp/ALLFILES.html$$", "ALLFILES.html");
    close (TEXT);
    &replace ("/tmp/ALLFILES$$", "ALLFILES");
}

sub dirindex
{
    ($workdir) = @_;
    local $basedir = $workdir;
    $workdir =~ s|^/pub|$rootpwd|go;
    chdir $workdir || die "Cannot chdir to $workdir!\n";

    open(HTML, ">/tmp/ALLDIRS.html$$") ||
	die "Could not create /tmp/ALLDIRS.html$$!\n";

    open(TEXT, ">/tmp/ALLDIRS$$") ||
	die "Could not create /tmp/ALLDIRS$$!\n";

    my ($uri) = '../' x (($basedir =~ tr|/|/|) - 2);
    print HTML '<?xml version="1.0"?><!-- -*- xml -*- -->
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
 "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html
 xmlns="http://www.w3.org/1999/xhtml"
 lang="en" xml:lang="en" dir="ltr">

<head>
<title>', &htmlname($basedir), ' tree</title>
<link rel="stylesheet" href="', $uri,
'.index.css" />
<link rel="icon" href="', $uri,
'.icon.png" />
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />
</head><body>

<h1>Directories under <a href="index.html">', &htmlname($basedir), "</a></h1>
<div class=\"dirs\">\n";

    print TEXT "Directories under $basedir\n\n";
    print TEXT "The HTML version is in $basedir/ALLDIRS.html\n\n\n";

    &processdirindex("", $workdir, 0);

    print HTML "</div>
<p class=\"bot\"><a href=\"${uri}MIRRORS.html\">Mirror sites</a>\n&ndash;
<a href=\"${uri}README.html\">General information</a>\n&ndash;
<a href=\"${uri}FILETYPES.html\">File types</a></p>
</body></html>\n";
    close (HTML);
    &replace ("/tmp/ALLDIRS.html$$", "ALLDIRS.html");
    close (TEXT);
    &replace ("/tmp/ALLDIRS$$", "ALLDIRS");
}

sub processdir
{
    local(%dirindex) = ();
    local(%fileindex) = ();
    local($cwd, $pwd, $indent) = @_;
    $comments = '';
    $commformat = ' ' x $indent . "* ^" . '<' x ($COLS - $indent - 2) .	"~~\n";
    local $descformat = ' ' x $indent . '- ^' .
	'<' x ($COLS - $indent - 2) . "~~\n";

    if(open(INDEX, '00INDEX'))
    {
	$name = '';
	$info = '';
	while(<INDEX>)
	{
	    chop while(/[\n\r]$/);
	    if(/^\*\*.*$/)
	    {
		$comments .= $_ . "\n";
	    }
	    else
	    {
		if(/^(\S.*)\s*$/)
		{
		    if($name ne '')
		    {
			&addentry($name, $info);
		    }
		    $name = $1;
		    $info = "\n";
		}
		elsif(/^\s+(\S.*)$/)
		{
		    if($info eq "\n")
		    {
			$info = '  ' . $1 . "\n";
		    }
		    else
		    {
			$info .= '  ' . $1 . "\n";
		    }
		}
		elsif($info !~ /\n\n$/)
		{
		    $info .= "\n";
		}
	    }
	}
	if($name ne '')
	{
	    &addentry($name, $info);
	}
	close(INDEX);
    }

    @keys = keys(%fileindex);
    if ($#keys >= 0 || $comments ne '')
    {
	print HTML "<dl>\n";
    }

    $tmp = $comments;
#Strip off the '**'s at line beginnigns.
#Add an empty line only if the previous line ends in one or more whitespaces.
    $tmp =~ s/^\*\*\s*//g;
    $tmp =~ s/\n\*\*\s*\n/\n\n/g;
    $tmp =~ s/\n\*\* */\n/g;
    $tmp =~ s/\s+\n/\n\n/g;

    &writetext($commformat, $tmp);

#Replace the <, > and & signs with their HTML equivalents.
    $tmp = &htmlname($comments);
#Generate a <br /> tag for a newline only for plain "**" lines
#or if the previous line ends in one or more whitespaces.
    $tmp =~ s|\n\*\*\n|<br />\n|g;
    $tmp =~ s|\s+\n\*\*\s*|<br />\n|g;
#Strip off the '**'s and white space at line beginnigns.
    $tmp =~ s/^\*\*\s*//g;
    $tmp =~ s/\n\*\*\s*/\n/g;
#Strip off trailing newlines.
    $tmp =~ s/\n+$//;

#Generate links for URLs.
    $tmp =~ s/$URLexpression/<a class="link" href="\1">\1<\/a>/go;

    if($tmp ne '')
    {
	print HTML "<dd>$tmp</dd>\n";
    }

    for(sort(keys(%fileindex)))
    {
	$tmp = $_;

	if (-l $_)
	{
	    $tmp2 = &resolvelink($_,$pwd);
	    print HTML "<dt><a href=\"", &urlname($tmp2), "\">",
	    	       &htmlname($tmp), "@</a> (link)</dt>\n";
	}
	else
	{
	    print HTML "<dt><a href=\"", &urlname($cwd . $_), "\">",
                       &htmlname($tmp), "</a></dt>\n";
	}

        print TEXT ' ' x $indent . "$tmp\n";

	&writedesc($fileindex{$tmp});
    }

    if ($#keys >= 0 || $comments ne '')
    {
	print HTML "</dl>\n";
    }

    @keys = keys(%dirindex);

    if ($#keys >= 0) {
	print HTML "<dl>\n";

	for(sort(keys(%dirindex)))
	{
	    $tmp = $_;
	    chop $tmp;

	    if ($tmp ne "incoming" && -l $tmp)
	    {
		$linktmp = &resolvelink($tmp,$pwd);
		print HTML "<dt><a href\=\"", &urlname($linktmp),
		      "/index.html\">",
		      &htmlname($tmp), "\@</a> (link)</dt>\n";
		print TEXT ' ' x $indent . "$cwd$tmp\@ -> $linktmp\n";
	    }
	    else
	    {
		print HTML "<dt><a href\=\"", &urlname($cwd . $tmp),
		      "/index.html\">", &htmlname($tmp), "/</a></dt>\n";
		print TEXT ' ' x $indent . "$cwd$tmp/\n";
	    }

	    &writedesc($dirindex{$_});

#recurse into the directory
	    if (!(-l $tmp) && ($tmp ne "incoming"))
	    {
		chdir "$pwd/$tmp" || die "Can't cd to $pwd/$tmp!\n";
		print HTML "<dd>";
		&processdir("$cwd$tmp/", "$pwd/$tmp", $indent + 2);
		print HTML "</dd>\n";
		chdir $pwd || die "Can't cd to $pwd!\n";
	    }
	}

	print HTML "</dl>\n";
    }
}

sub processdirindex
{
    local(%dirindex) = ();
    local($cwd, $pwd, $indent) = @_;
    $comments = '';
    $commformat = ' ' x $indent . "* ^" . '<' x ($COLS - $indent - 2) .	"~~\n";
    local $descformat = ' ' x $indent . '- ^' .
	'<' x ($COLS - $indent - 2) . "~~\n";

    if(open(INDEX, '00INDEX'))
    {
	$name = '';
	$info = '';
	while(<INDEX>)
	{
	    chop while(/[\n\r]$/);
	    if(/^\*\*.*$/)
	    {
		$comments .= $_ . "\n";
	    }
	    else
	    {
		if(/^(\S.*)\s*$/)
		{
		    if($name ne '')
		    {
			&addentry($name, $info);
		    }
		    $name = $1;
		    $info = "\n";
		}
		elsif(/^\s+(\S.*)$/)
		{
		    if($info eq "\n")
		    {
			$info = '  ' . $1 . "\n";
		    }
		    else
		    {
			$info .= '  ' . $1 . "\n";
		    }
		}
		elsif($info !~ /\n\n$/)
		{
		    $info .= "\n";
		}
	    }
	}
	if($name ne '')
	{
	    &addentry($name, $info);
	}
	close(INDEX);
    }

    $tmp = $comments;
#Strip off the '**'s at line beginnigns.
#Add an empty line only if the previous line ends in one or more whitespaces.
    $tmp =~ s/^\*\*\s*//g;
    $tmp =~ s/\n\*\*\s*\n/\n\n/g;
    $tmp =~ s/\n\*\* */\n/g;
    $tmp =~ s/\s+\n/\n\n/g;

    &writetext($commformat, $tmp);

#Replace the <, > and & signs with their HTML equivalents.
    $tmp = &htmlname($comments);
#Generate a <br /> tag for a newline only for plain "**" lines
#or if the previous line ends in one or more whitespaces.
    $tmp =~ s|\n\*\*\n|<br />\n|g;
    $tmp =~ s|\s+\n\*\*\s*|<br />\n|g;
#Strip off the '**'s and any white spaces at line beginnigns.
    $tmp =~ s/^\*\*\s*//g;
    $tmp =~ s/\n\*\* */\n/g;
#Strip off any trailing newlines.
    $tmp =~ s/\n+$//;

#Generate links for URLs.
    $tmp =~ s/$URLexpression/<a class="link" href="\1">\1<\/a>/go;

    @keys = keys(%dirindex);

    if($tmp ne '')
    {
	print HTML "<dl>\n<dd>$tmp</dd>\n";

	if ($#keys < 0) {
	    print HTML "</dl>\n";
	}
    }
    elsif ($#keys >= 0)
    {
	print HTML "<dl>\n";
    }

    if ($#keys >= 0) {
	for(sort(keys(%dirindex)))
	{
	    $tmp = $_;
	    chop $tmp;
	    if (-l $tmp)
	    {
		$linktmp = &resolvelink($tmp,$pwd);
		print HTML "<dt><a href\=\"", &urlname($linktmp),
		      "/index.html\">",
		      &htmlname($tmp), "\@</a> (link)</dt>\n";
		print TEXT ' ' x $indent . "$cwd$tmp\@ -> $linktmp\n";
	    }
	    else
	    {
		print HTML "<dt><a href\=\"", &urlname($cwd . $tmp),
		      "/index.html\">", &htmlname($tmp), "/</a></dt>\n";
		print TEXT ' ' x $indent . "$cwd$tmp/\n";
	    }

	    &writedesc($dirindex{$_});

#recurse into the directory
	    if (!(-l $tmp) && ($tmp ne "incoming"))
	    {
		chdir "$pwd/$tmp" || die "Can't cd to $pwd/$tmp!\n";
		print HTML "<dd>";
		&processdirindex("$cwd$tmp/", "$pwd/$tmp", $indent + 2);
		print HTML "</dd>\n";
		chdir $pwd || die "Can't cd to $pwd!\n";
	    }
	}

	print HTML "</dl>\n";
    }
}

sub addentry
{
    local($name, $info) = @_;

#remove the nonsense descriptions "[no description]" and "--"
    $info =~ s/^\s*(\[no description\]|--)\s*\n+$//;

    if ($name =~ /\/$/)
    {
	# directory
	$dirindex{$name} = $info;
    }
    else
    {
	# file
	$fileindex{$name} = $info;
    }
}

sub writetext
{
    local ($format, $str) = @_;

    $str =~ s/\n\n+/\n\n/g;
    my @args = split (/\n\n/, $str);

    foreach $str (@args)
    {
	$^A = "";
	formline($format, $str);
	print TEXT $^A;
    }
}

sub writedesc
{
    my $text = shift;
    my $tmp = $text;

#Strip off any leading whitespaces and trailing newlines in the comments.
#Add an empty line only if the previous line ends in one or more whitespaces.
    $text =~ s/^\s*//;
    $text =~ s/\s+\n\s*/\n\n/g;
    $text =~ s/\n +/\n/g;

    &writetext($descformat, $text);

#Replace the <, > and & signs with their HTML equivalents.
    $tmp =~ s/&/&amp\;/g;
    $tmp =~ s/</&lt\;/g;
    $tmp =~ s/>/&gt\;/g;
#Strip off any leading whitespaces and trailing newlines in the comments.
#Generate a <br /> tag for a newline only if
#the previous line ends in one or more whitespaces.
    $tmp =~ s/^\s*//;
    $tmp =~ s/\n+$//;
    $tmp =~ s|\s+\n\s*|<br />\n|g;
    $tmp =~ s/\n\s*/\n/g;

#Generate links for URLs.
    $tmp =~ s/$URLexpression/<a class="link" href="\1">\1<\/a>/go;

    if($tmp ne "")
    {
	print HTML "<dd>$tmp</dd>\n";
    }
}

sub urlname
{
    ($_) = @_;
    s/([\000-\040\"\#%\?:+&<>\176-\377])/"%".unpack("H2",$1)/ges;
    return $_;
}

sub htmlname
{
    ($_) = @_;
#Replace the <, > and & signs with their HTML equivalents.
    s/&/&amp\;/g;
    s/</&lt\;/g;
    s/>/&gt\;/g;
    return $_;
}

sub resolvelink
{
    my($name,$pwd) = @_;
    $name = res_link($name,$pwd);
    if ($name !~ /\// && (-f $name)) {
	my($dir) = `pwd`; chop $dir;
	$name = &truncpath($dir.'/'.$name,$workdir);
    }
    chdir $pwd || die "could not chdir back to `$pwd'\n";
    return $name;
}

sub res_link
{
    my($name,$pwd) = @_;
    if (!(-l $name)) {
	(-f $name) || warn "res_link called for `$name' in `$pwd'\n";
	return $name;
    }
    $name = readlink($name);
    if (-f $name) {
	my($dir) = $name;
	$dir =~ s|/([^/]*)$|| || return &res_link($name,$pwd);
	($dir ne "") || return &res_link($1,$pwd);
	my($file) = $1;
	chdir $dir || die "could not chdir to `$dir'\n";
	$dir=`pwd`;chop $dir;
	$name = &truncpath($dir,$workdir).'/'.&res_link($file,$dir);
	return $name;
    }
    elsif (-d $name) {
	chdir $name || die "could not chdir to `$name'\n";
	$name=`pwd`;chop $name;
	return &truncpath($name,$workdir);
    }
    else {
	warn "`$name' neither file or directory\n";
	return $name;
    }
}

sub truncpath ()
{
    my($path,$prefix) = @_;

    $path =~ s|^\Q$prefix\E/|| && return $path;

    my($dir) = $path;
    while ($prefix !~ s|^\Q$dir/\E||) {
	$dir =~ s|/[^/]*$|| ||
	    die "truncpath (`$path', `$prefix') failed\n";
    }
    $path =~ s|^\Q$dir/\E|| ||
	die "could not truncate `$path' with `$dir'\n";
    for (split ('/', $prefix)) {
	$path = '../' . $path;
    }

    return $path;
}
